@@ -1,14 +1,15 @@
 from __future__ import absolute_import, division, print_function
 
 import os
+import weakref
 import platform
+from collections import OrderedDict
 
 import numpy as np
 from matplotlib.colors import ColorConverter
 
 from qtpy import QtWidgets
 from glue.core.message import SettingsChangeMessage
-from glue.utils import nonpartial
 from glue.utils.qt import load_ui, ColorProperty, get_qapp
 from glue.utils.qt.widget_properties import (CurrentComboTextProperty,
                                              ValueProperty, ButtonProperty)
@@ -18,6 +19,45 @@ __all__ = ["PreferencesDialog"]
 
 rgb = ColorConverter().to_rgb
 
+AUTOLINK_OPTIONS = OrderedDict([
+    ('always_show', 'Always show suggestions'),
+    ('always_accept', 'Always accept suggestions'),
+    ('always_ignore', 'Always ignore suggestions')
+])
+
+
+class AutolinkPreferencesPane(QtWidgets.QWidget):
+
+    def __init__(self, parent=None):
+        super(AutolinkPreferencesPane, self).__init__(parent=parent)
+
+        from glue.config import settings, autolinker  # noqa
+
+        layout = QtWidgets.QGridLayout()
+
+        self.combos = {}
+
+        for i, (label, _) in enumerate(autolinker):
+            combo = QtWidgets.QComboBox()
+            for short, display in AUTOLINK_OPTIONS.items():
+                combo.addItem(display, userData=short)
+            if label in settings.AUTOLINK:
+                index = list(AUTOLINK_OPTIONS.keys()).index(settings.AUTOLINK[label])
+            else:
+                index = 0
+            combo.setCurrentIndex(index)
+            layout.addWidget(QtWidgets.QLabel(label), i, 0)
+            layout.addWidget(combo, i, 1)
+            self.combos[label] = combo
+        layout.addWidget(QtWidgets.QWidget(), i + 1, 0)
+
+        self.setLayout(layout)
+
+    def finalize(self):
+        from glue.config import settings
+        for label, combo in self.combos.items():
+            settings.AUTOLINK[label] = combo.currentData()
+
 
 class PreferencesDialog(QtWidgets.QDialog):
 
@@ -27,14 +67,14 @@ class PreferencesDialog(QtWidgets.QDialog):
     data_color = ColorProperty('ui.color_default_data')
     data_alpha = ValueProperty('ui.slider_alpha', value_range=(0, 1))
     data_apply = ButtonProperty('ui.checkbox_apply')
-    show_large_data_warning = ButtonProperty('ui.checkbox_show_large_data_warning')
     save_to_disk = ButtonProperty('ui.checkbox_save')
+    font_size = ValueProperty('ui.spinner_font_size')
 
     def __init__(self, application, parent=None):
 
         super(PreferencesDialog, self).__init__(parent=parent)
 
-        self.app = application
+        self._app = weakref.ref(application)
 
         self.ui = load_ui('preferences.ui', self,
                           directory=os.path.dirname(__file__))
@@ -42,7 +82,9 @@ class PreferencesDialog(QtWidgets.QDialog):
         self.ui.cancel.clicked.connect(self.reject)
         self.ui.ok.clicked.connect(self.accept)
 
-        self.ui.combo_theme.currentIndexChanged.connect(nonpartial(self._update_colors_from_theme))
+        self.ui.combo_theme.currentIndexChanged.connect(self._update_colors_from_theme)
+
+        self.ui.button_reset_dialogs.clicked.connect(self._reset_dialogs)
 
         # The following is needed because of a bug in Qt which means that
         # tab titles don't get scaled right.
@@ -56,10 +98,13 @@ class PreferencesDialog(QtWidgets.QDialog):
         self.foreground = settings.FOREGROUND_COLOR
         self.data_color = settings.DATA_COLOR
         self.data_alpha = settings.DATA_ALPHA
-        self.show_large_data_warning = settings.SHOW_LARGE_DATA_WARNING
+        self.font_size = settings.FONT_SIZE
 
         self._update_theme_from_colors()
 
+        self._autolink_pane = AutolinkPreferencesPane()
+        self.ui.tab_widget.addTab(self._autolink_pane, 'Autolinking')
+
         self.panes = []
 
         from glue.config import preference_panes
@@ -68,17 +113,17 @@ class PreferencesDialog(QtWidgets.QDialog):
             self.ui.tab_widget.addTab(pane, label)
             self.panes.append(pane)
 
-    def _update_theme_from_colors(self):
-        if (rgb(self.background) == (1, 1, 1) and rgb(self.foreground) == (0, 0, 0)
-                and rgb(self.data_color) == (0.35, 0.35, 0.35) and np.allclose(self.data_alpha, 0.8)):
+    def _update_theme_from_colors(self, *args):
+        if (rgb(self.background) == (1, 1, 1) and rgb(self.foreground) == (0, 0, 0) and
+                rgb(self.data_color) == (0.35, 0.35, 0.35) and np.allclose(self.data_alpha, 0.8)):
             self.theme = 'Black on White'
-        elif (rgb(self.background) == (0, 0, 0) and rgb(self.foreground) == (1, 1, 1)
-              and rgb(self.data_color) == (0.75, 0.75, 0.75) and np.allclose(self.data_alpha, 0.8)):
+        elif (rgb(self.background) == (0, 0, 0) and rgb(self.foreground) == (1, 1, 1) and
+                rgb(self.data_color) == (0.75, 0.75, 0.75) and np.allclose(self.data_alpha, 0.8)):
             self.theme = 'White on Black'
         else:
             self.theme = 'Custom'
 
-    def _update_colors_from_theme(self):
+    def _update_colors_from_theme(self, *args):
         if self.theme == 'Black on White':
             self.foreground = 'black'
             self.background = 'white'
@@ -92,6 +137,12 @@ class PreferencesDialog(QtWidgets.QDialog):
         elif self.theme != 'Custom':
             raise ValueError("Unknown theme: {0}".format(self.theme))
 
+    def _reset_dialogs(self, *args):
+        from glue.config import settings
+        for key, _, _ in settings:
+            if key.lower().startswith(('show_info', 'show_warn', 'show_large')):
+                setattr(settings, key, True)
+
     def accept(self):
 
         # Update default settings
@@ -101,7 +152,9 @@ class PreferencesDialog(QtWidgets.QDialog):
         settings.BACKGROUND_COLOR = self.background
         settings.DATA_COLOR = self.data_color
         settings.DATA_ALPHA = self.data_alpha
-        settings.SHOW_LARGE_DATA_WARNING = self.show_large_data_warning
+        settings.FONT_SIZE = self.font_size
+
+        self._autolink_pane.finalize()
 
         for pane in self.panes:
             pane.finalize()
@@ -109,22 +162,23 @@ class PreferencesDialog(QtWidgets.QDialog):
         # Save to disk if requested
         if self.save_to_disk:
             save_settings()
+        else:
+            settings._save_to_disk = True
 
         # Trigger viewers to update defaults
 
-        self.app._hub.broadcast(SettingsChangeMessage(self, ('FOREGROUND_COLOR', 'BACKGROUND_COLOR')))
+        app = self._app()
 
-        # If requested, trigger data to update color
-
-        if self.data_apply:
-            self.app.set_data_color(settings.DATA_COLOR, settings.DATA_ALPHA)
+        if app is not None:
+            app._hub.broadcast(SettingsChangeMessage(self, ('FOREGROUND_COLOR', 'BACKGROUND_COLOR', 'FONT_SIZE')))
+            if self.data_apply:  # If requested, trigger data to update color
+                app.set_data_color(settings.DATA_COLOR, settings.DATA_ALPHA)
 
         super(PreferencesDialog, self).accept()
 
 
 if __name__ == "__main__":
 
-    from glue.utils.qt import get_qapp
     app = get_qapp()
     widget = PreferencesDialog()
     widget.show()
